from natsort import natsorted
import os,cv2
from pathlib import Path

from pyCAIR.helpers import createFolder as cF

def vid(path):

	dir_path = path
	ext1, ext2 = '.png', '.jpg'
	opath = str(Path(__file__).resolve().parents[0]) + '\\videos'
	cF(opath)	
	a, b = dir_path.rsplit('\\', 1)[0], dir_path.rsplit('\\', 1)[1]
	c, d = a.rsplit('\\', 1)[0], a.rsplit('\\', 1)[1]
	e, f = c.rsplit('\\', 1)[0], c.rsplit('\\', 1)[1]
	vid_name = f + '_' + d + '_' + b + '.avi'
	print(vid_name)
	op = os.path.join(opath, vid_name)

	#exit()
	shape = 640, 540
	fps = 5

	#images = [f for f in os.listdir(dir_path) if f.endswith(ext)]
	images = []
	for f in os.listdir(dir_path):
		if f.endswith(ext1) or f.endswith(ext2):
			images.append(f)

	images = natsorted(images)
	print(images[0])

	fourcc = cv2.VideoWriter_fourcc(*'DIVX')
	video = cv2.VideoWriter(op, fourcc, fps, shape)

	for image in images:
	    image_path = os.path.join(dir_path, image)
	    image = cv2.imread(image_path)
	    resized=cv2.resize(image,shape) 
	    video.write(resized)
	video.release()

def getToProcessPaths(directory):
	
	all_subdirs = [x[0] for x in os.walk(directory)]
	get = []
	for i in range(len(all_subdirs)):
		if all_subdirs[i].endswith('cropseq') or all_subdirs[i].endswith('seamseq'):
			if all_subdirs[i] not in get:
				get.append(all_subdirs[i])

	return get

def generateVideo():
	base_path = str(Path(__file__).resolve().parents[0])
	base_path += "\sequences\\"
	allpaths = getToProcessPaths(base_path)

	for i in range(len(allpaths)):
		cpath = allpaths[i]
		vid(cpath)


from .notdoneyet import user_input

from .opencv_generators import generateEnergyMap
from .opencv_generators import generateColorMap

from .imgtovideos import generateVideo
from .imgtovideos import getToProcessPaths

from .helpers import createFolder
from .helpers import getFileExtension
from .helpers import writeImage

from .seam_carve import cropByColumn
from .seam_carve import cropByRow


import cv2
import numpy as np

from pyCAIR.helpers import writeImage as wI

def generateEnergyMap(image, file_extension, file_name):
	image = cv2.cvtColor(image.astype(np.uint8), cv2.COLOR_BGR2GRAY)
	wI(image, ['gray', file_extension, file_name])
	dx = cv2.Sobel(image, cv2.CV_16S, 1, 0, ksize=3)
	abs_x = cv2.convertScaleAbs(dx)
	dy = cv2.Sobel(image, cv2.CV_16S, 0, 1, ksize=3)
	abs_y = cv2.convertScaleAbs(dy)
	output = cv2.addWeighted(abs_x, 0.5, abs_y, 0.5, 0)
	wI(output, ['energy', file_extension, file_name])

def generateColorMap(image, file_extension, file_name):
	img = image
	gray_img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
	heatmap1_img = cv2.applyColorMap(gray_img, 3)
	heatmap2_img = cv2.applyColorMap(gray_img, 11)
	superimpose1 = cv2.addWeighted(heatmap1_img, 0.7, img, 0.3, 0)
	superimpose2 = cv2.addWeighted(heatmap2_img, 0.7, img, 0.3, 0)
	wI(superimpose1, ['colormap1', file_extension, file_name])
	wI(superimpose2, ['colormap2', file_extension, file_name])


import os, cv2

def createFolder(directory):
	if not os.path.exists(directory):
		os.makedirs(directory)

def getFileExtension(ip):
	front, back = ip.split('.')
	folder, name = front.split('/')
	return back, name

def writeImage(image, args):
	name = 'results/' + str(args[2]) + '/' + str(args[0]) + '.' + str(args[1])
	cv2.imwrite(name, image)
	cv2.destroyAllWindows()

def writeImageG(image, cname, extension, filename, switch, _path = 'col-wise'):
	if switch == 0:
		insert = 'cropseq'
	else:
		insert = 'seamseq'

	name = 'sequences/' + filename + '/' + _path + '/' + insert + '/' + cname + '.' + extension
	cv2.imwrite(name, image)


from distutils.core import setup
import setuptools

with open('README.rst') as file:
    long_description = file.read()

python_requires = '>3.4'
    
setup(
  name = 'pyCAIR',
  packages = ['pyCAIR'],
  version = '0.1.11',
  description = ' This module provides a simple yet powerful mechanism to resize images using Seam Carving Algorithm.',
  long_description = long_description,
  long_description_content_type="text/markdown",
  author = 'Chirag Shah',
  author_email = 'chiragshah9696@gmail.com',
  url = 'https://github.com/avidLearnerInProgress/pyCAIR',
  download_url = 'https://github.com/avidLearnerInProgress/pyCAIR/archive/0.1.tar.gz',
  license = 'GPL-3.0',
  keywords = ['python', 'openCV', 'image-processing', 'dynamic-programming', 'seam-carving'],
  classifiers = [],
)


import os
import numpy as np
from scipy.ndimage import rotate
from scipy.ndimage.filters import convolve
from tqdm import trange

from pyCAIR.helpers import writeImage as wI
from pyCAIR.helpers import createFolder as cF
from pyCAIR.helpers import writeImage as wI
from pyCAIR.helpers import writeImageG as wIG

def getEnergy(image):
	
	filter_x = np.array([
		[1.0, 2.0, 1.0],
		[0.0, 0.0, 0.0],
		[-1.0, -2.0, -1.0],
		])

	filter_x = np.stack([filter_x] * 3, axis = 2)

	filter_y = np.array([
		[1.0, 0.0, -1.0],
		[2.0, 0.0, -2.0],
		[1.0, 0.0, -1.0],
		])

	filter_y = np.stack([filter_y] * 3, axis = 2)

	image = image.astype('float32')

	convoluted = np.absolute(convolve(image, filter_x)) + np.absolute(convolve(image, filter_y))

	energy_map = convoluted.sum(axis = 2)

	return energy_map

def getMaps(image):
	rows, columns, _ = image.shape
	energy_map = getEnergy(image)

	current_map = energy_map.copy()
	goback = np.zeros_like(current_map, dtype = np.int)

	for i in range(1, rows):
		for j in range(0, columns):
			if j == 0:
				min_index = np.argmin(current_map[i - 1, j : j + 2])
				goback[i, j] = min_index + j
				min_energy = current_map[i - 1, min_index + j]

			else:
				min_index = np.argmin(current_map[i - 1, j - 1 : j + 2])
				goback[i, j] = min_index + j -1
				min_energy = current_map[i - 1, min_index + j - 1]

			current_map[i, j] += min_energy

	return current_map, goback

def drawSeam(image):

	rows, columns, _ = image.shape
	cMap, goback = getMaps(image)

	mask = np.ones((rows, columns), dtype = np.bool)

	j = np.argmin(cMap[-1])

	for i in reversed(range(rows)):
		mask[i, j] = False
		j = goback[i, j]

	mask = np.logical_not(mask)
	image[...,0][mask] = 0 
	image[...,1][mask] = 0
	image[...,2][mask] = 255

	return image

def carve(image):

	rows, columns, _ = image.shape
	cMap, goback = getMaps(image)

	mask = np.ones((rows, columns), dtype = np.bool)

	j = np.argmin(cMap[-1])

	for i in reversed(range(rows)):
		mask[i, j] = False
		j = goback[i, j]

	mask = np.stack([mask] * 3, axis = 2)
	image = image[mask].reshape((rows, columns - 1, 3))
	
	return image

def cropByColumn(image, display_seams, generate = 0, lsit = None, scale_c = 0.5, fromRow = 0):

	rows, columns, _ = image.shape

	newcolumns = int(columns * scale_c)
	crop = image.copy()

	if fromRow == 1:
		_path = 'row-wise'
	else:
		_path = 'col-wise'

	if display_seams == 0:
		a = 0
		gc = 0
		cF(os.getcwd() + str('\\sequences\\' + lsit[0] + '\\' + _path + '\\cropseq\\'))
		for i in trange(columns - newcolumns):
			if generate == 1:
				crop = carve(crop)
				if i % 5 == 0:
					if fromRow == 1:
						_rotate = crop.copy()
						_rotate = np.rot90(_rotate, 3, (0, 1))
						wIG(_rotate, str(gc)+'. cropped_'+str(i), lsit[1], lsit[0], a, _path)
						gc += 1
					else:
						wIG(crop, str(gc)+'. cropped_'+str(i), lsit[1], lsit[0], a)
						gc += 1
				else:
					pass
			else:
				crop = carve(crop)

		return crop

	else:
		a = 0
		b = 1
		gc_img = 0
		gc_crop = 0
		cF(os.getcwd() + str('\\sequences\\' + lsit[0] + '\\' + _path + '\\cropseq\\'))
		cF(os.getcwd() + str('\\sequences\\' + lsit[0] + '\\' + _path + '\\seamseq\\'))
		for i in trange(columns - newcolumns):
			if generate == 1:
				#give me a way to parallelize this portion of code :|
				image = drawSeam(image) 
				crop = carve(crop)
				if i % 5 == 0:	
					if fromRow == 1:
						_rotate1 = image.copy()
						_rotate2 = crop.copy()
						_rotate1 = np.rot90(_rotate1, 3, (0, 1))
						_rotate2 = np.rot90(_rotate2, 3, (0, 1))
						wIG(_rotate1, str(gc_img)+'. seamed_'+str(i), lsit[1], lsit[0], b, _path)
						wIG(_rotate2, str(gc_crop)+'. cropped_'+str(i), lsit[1], lsit[0], a, _path)
						gc_img += 1
						gc_crop += 1
					else:
						wIG(image,str(gc_img)+'. seamed_'+str(i), lsit[1], lsit[0], b)
						wIG(crop,str(gc_crop)+'. cropped_'+str(i), lsit[1], lsit[0], a)
						gc_img += 1
						gc_crop += 1
				else:
					pass
			else:
				image = drawSeam(image) 
				crop = carve(crop)
				
		return image, crop

def cropByRow(image, display_seams, generate = 0, lsit = None, scale_r = 0.5):

	fromRow = 1
	image = np.rot90(image, 1, (0, 1))
	seam_image, crop_image = cropByColumn(image, display_seams, generate, lsit, scale_r, fromRow)
	crop_image = np.rot90(crop_image, 3, (0, 1))
	seam_image = np.rot90(seam_image, 3, (0, 1))

	return seam_image, crop_image



import os, cv2

#Local imports
from pyCAIR.imgtovideos import generateVideo as gV
from pyCAIR.helpers import getFileExtension as gFE
from pyCAIR.helpers import createFolder as cF
from pyCAIR.helpers import writeImage as wI
from pyCAIR.opencv_generators import generateEnergyMap as gEM
from pyCAIR.opencv_generators import generateColorMap as gCM
from pyCAIR.seam_carve import cropByColumn as cBC
from pyCAIR.seam_carve import cropByRow as cBR

def main(argsip):
	#usr inpt
	toggle = argsip[0]
	scale = argsip[1]
	display_seams = argsip[2]
	_in = argsip[3]
	g = argsip[4]

	image = cv2.imread(_in)
	file_extension, file_name = gFE(_in)
	#print(file_extension + " " + file_name)
	root = os.getcwd() + str('\\results\\')
	cF(root + file_name)
	gEM(image, file_extension, file_name)
	gCM(image, file_extension, file_name)
	image_ = image.copy()
	lsit = [file_name, file_extension]

	if toggle == 0:
		#cropbycol
		if display_seams == 1:
			seam_image, crop_image = cBC(image, display_seams, g, lsit, scale)
			wI(seam_image, ['column_seams', file_extension, file_name])
			wI(crop_image, ['column_cropped', file_extension, file_name])
			
		else:
			crop_image = cBC(image, display_seams, g, lsit, scale)
			wI(crop_image, ['column_cropped', file_extension, file_name])

	elif toggle == 1:
		#cropbyrow
		if display_seams == 1:
			seam_image, crop_image = cBR(image, display_seams, g, lsit, scale)
			wI(seam_image, ['row_seams', file_extension, file_name])
			wI(crop_image, ['row_cropped', file_extension, file_name])

		else:
			crop_image = cBR(image, display_seams, g, lsit, scale)
			wI(crop_image, ['row_cropped', file_extension, file_name])

	elif toggle == 2:
		#cropbyrow&column
		if display_seams == 1:
			seam_col, crop_col = cBC(image, display_seams, g, lsit, scale)
			seam_row, crop_row = cBR(image_, display_seams, g, lsit, scale)
			wI(seam_col, ['column_seams', file_extension, file_name])
			wI(seam_row, ['row_seams', file_extension, file_name])
			wI(crop_col, ['column_cropped', file_extension, file_name])
			wI(crop_row, ['row_cropped', file_extension, file_name])
			
		else:
			crop_col = cBC(image, display_seams, g, scale)
			crop_row = cBR(image, display_seams, g, scale)
			wI(crop_row, ['row_cropped', file_extension, file_name])
			wI(crop_col, ['column_cropped', file_extension, file_name])
	else:
		print('Invalid input!')
		exit()

	gV()

def user_input(alignment, scale , display_seam, image, generate):
	'''parser = argparse.ArgumentParser()
	parser.add_argument('-a',
						type = int,
						help = "Alignment of Seam (Vertical/Horizontal/Both) : (0/1/2)",
						required = True)

	parser.add_argument('-s',
						type = float,
						help = "Scale aspect ratio between 0 and 1",
						default = 0.5,
						required = False)

	parser.add_argument('-d',
						type = int,
						help = "Display Seams or not : (1/0)",
						required = True)

	parser.add_argument('-i',
						type = str,
						help = "Path to source image",
						required = True)

	parser.add_argument('-g',
						type = int,
						help = "Generate sequences of image or not : (1/0)",
						required = False)

	argsip = parser.parse_args()
	'''
	argsip = []
	argsip.append(alignment)
	argsip.append(scale)
	argsip.append(display_seam)
	argsip.append(image)
	argsip.append(generate)
	main(argsip)